"use strict";

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var ShortcodeFormatter = require('./shortcode-formatter');

/**
 * Data structure that represents a single [shortcode].
 */

var Shortcode = function () {
  function Shortcode(name, content, properties, isSelfClosing, codeText, offset) {
    _classCallCheck(this, Shortcode);

    /**
     * The tag name.
     *
     * @type {null|string}
     */
    this.name = name || null;

    /**
     * Raw content contained within this shortcode.
     * Will be NULL for self-closing tags.
     *
     * @type {null|string}
     */
    this.content = typeof content === "undefined" ? null : content;

    /**
     * Key/value object that holds all properties for this shortcode.
     *
     * @type {object}
     */
    this.properties = properties || {};

    /**
     * Indicates whether this is a self-closing tag or not.
     *
     * @type {boolean}
     */
    this.isSelfClosing = !!isSelfClosing;

    /**
     * Raw text block that was parsed for this shortcode.
     *
     * @type {string|null}
     */
    this.codeText = codeText || null;

    /**
     * Offset, relative to the extractor input string.
     *
     * @type {number}
     */
    this.offset = offset || 0;
  }

  /**
   * Internal helper function: Get the end offset of this shortcode, relative to the extractor input string.
   *
   * @returns {*}
   */


  _createClass(Shortcode, [{
    key: "getEndOffset",
    value: function getEndOffset() {
      return this.offset + this.codeText.length;
    }

    /**
     * Gets whether this shortcode has a set property with a given `key`.
     *
     * @param {string} key
     * @returns {boolean}
     */

  }, {
    key: "hasProperty",
    value: function hasProperty(key) {
      return this.properties && typeof this.properties[key] !== "undefined";
    }

    /**
     * Get the value of a property.
     *
     * @param key
     * @returns {string|null}
     */

  }, {
    key: "getProperty",
    value: function getProperty(key) {
      return this.hasProperty(key) ? this.properties[key] : null;
    }

    /**
     * Add or update a property.
     *
     * @param {string} key
     * @param value
     */

  }, {
    key: "setProperty",
    value: function setProperty(key, value) {
      this.properties[key] = value;
    }

    /**
     * Appends a given `shortcode` to this shortcode's content, by rendering the input as a string.
     *
     * @param {Shortcode} shortcode
     */

  }, {
    key: "addChild",
    value: function addChild(shortcode) {
      this.appendContent(shortcode.stringify());
    }

    /**
     * Appends given `content` to the end of this shortcode's content.
     *
     * @param {string} content
     */

  }, {
    key: "appendContent",
    value: function appendContent(content) {
      if (!this.content) {
        this.content = "";
      }

      this.content += content;
    }

    /**
     * Renders this Shortcode as formatted code, in shortcode style.
     *
     * @returns {string}
     */

  }, {
    key: "stringify",
    value: function stringify() {
      return ShortcodeFormatter.stringify(this);
    }

    /**
     * Renders this Shortcode as formatted code, in HTML style.
     *
     * @param {string|null} tagName Optional HTML tag name override.
     * @param {object} options Formatter options
     * @return {*}
     */

  }, {
    key: "stringifyAsHtml",
    value: function stringifyAsHtml(tagName, options) {
      var optionsMerged = {
        asHtml: true,
        tagName: tagName ? tagName : null
      };

      if (options) {
        optionsMerged = Object.assign({}, options, optionsMerged);
      }

      return ShortcodeFormatter.stringify(this, optionsMerged);
    }

    /**
     * Converts this item to string.
     *
     * @returns {string}
     */

  }, {
    key: "toString",
    value: function toString() {
      return this.stringify();
    }
  }]);

  return Shortcode;
}();

module.exports = Shortcode;